unit NZCSFont;

{------------------------------------------------------------------------------}
{Copyright 1999, Greg Nixon.  All Rights Reserved.                             }
{This component can be freely used and distributed in commercial and private   }
{environments, provied this notice is not modified in any way and there is     }
{no charge for it other than nomial handling fees.  Contact me directly for    }
{modifications to this agreement. You cannot distributed modified versions     }
{and if you make changes please send me a copy so I can implement the change.  }
{                                                                              }
{This is provided as is, expressly without a warranty of any kind.             }
{You use it at your own risk.                                                  }
{                                                                              }
{Contact:                                                                      }
{Greg Nixon   greg@nzcs.co.nz                                                  }
{New Zealand Computing Solutions Ltd.                                          }
{------------------------------------------------------------------------------}

{Version 1.0.0.0 - Monday, 1 November 1999}

interface

uses Windows, Sysutils, Classes, Graphics, Dialogs;

Type
   TNZCSFontClass = class(TPersistent)
   private
      FFontName: String;
      FOnChange: TNotifyEvent;
      FFontSize: Smallint;
      FFontBold, FFontItalic, FFontUnderline, FFontStrikeOut: Boolean;
      FFontColor: TColor;
      FUpdateLock: Byte;
      Procedure SetFontName(Value: String);
      Procedure SetFontSize(Value: Smallint);
      Procedure SetFontColor(Value: TColor);
      Procedure SetFontBold(Value: Boolean);
      Procedure SetFontItalic(Value: Boolean);
      Procedure SetFontUnderline(Value: Boolean);
      procedure SetFontStrikeOut(Value: Boolean);
   public
      constructor Create;
      destructor Destroy; override;
      procedure Assign(Source: TPersistent); override;
      procedure BeginUpdate; virtual;
      procedure EndUpdate; virtual;
   published
      property OnChange: TNotifyEvent read FOnChange write FOnChange;
      property FontName: string read FFontName write SetFontName;
      property FontSize: Smallint read FFontSize write SetFontSize Default 12;
      property FontBold: Boolean read FFontBold write SetFontBold Default False;
      property FontItalic: Boolean read FFontItalic write SetFontItalic Default False;
      property FontUnderline: Boolean read FFontUnderline write SetFontUnderline Default False;
      property FontStrikeOut: Boolean read FFontStrikeOut write SetFontStrikeOut Default False;
      property FontColor: TColor read FFontColor write SetFontColor Default clWindowText;
   end;

   // Sample component with a Font property //
   TNZCSComponent = class(TComponent)
   private
      FFont: TNZCSFontClass;
      FOnFontChange: TNotifyEvent;
      Procedure FontChanged(Sender: TObject);
   public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
   published
      Property Font: TNZCSFontClass Read FFont write FFont;
      Property OnFontChange: TNotifyEvent Read FOnFontChange write FOnFontChange;
   end;

procedure Register;

implementation

uses DsgnIntf;

{ TNZCSFontEditor }

type
  TNZCSFontEditor = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure TNZCSFontEditor.Edit;
var
   FD: TFontDialog;
   FC: TNZCSFontClass;
begin
  FC := TNZCSFontClass(GetOrdValue);
  FD := TFontDialog.Create(nil);
  With FC do begin
     With FD.Font do begin
        Name := FC.FontName;
        Size := FC.FontSize;
        Color := FC.FontColor;
        Style := [];
        If FC.FontBold then
           Style := Style + [fsBold];
        If FC.FontItalic then
           Style := Style + [fsItalic];
        If FC.FontUnderline then
           Style := Style + [fsUnderline];
        If FC.FontStrikeOut then
           Style := Style + [fsStrikeOut];
     end;
  end;
  Try
     If FD.Execute then begin
        FC.Assign(FD.Font);
        Designer.modified;
     end;
  finally
     FD.Free;
  end;
end;

function TNZCSFontEditor.GetAttributes: TPropertyAttributes;
begin
  GetAttributes := [paMultiSelect,paSubProperties,paDialog, paReadOnly];
end;

{ TNZCSFontClass }

constructor TNZCSFontClass.Create;
begin
   Inherited Create;
   FFontName := 'Times New Roman';
   FFontSize := 12;
   FFontBold := False;
   FFontItalic := False;
   FFontUnderline := False;
   FFontColor := clWindowText;
   FUpdateLock := 0;
end;

destructor TNZCSFontClass.Destroy;
begin
   inherited Destroy;
end;

procedure TNZCSFontClass.Assign(Source: TPersistent);
begin
   BeginUpdate;
   Try
      if Source is TNZCSFontClass then begin
         FFontName := TNZCSFontClass(Source).FontName;
         FFontSize := TNZCSFontClass(Source).FontSize;
         FFontBold := TNZCSFontClass(Source).FontBold;
         FFontItalic := TNZCSFontClass(Source).FontItalic;
         FFontUnderline := TNZCSFontClass(Source).FontUnderline;
         FFontStrikeOut := TNZCSFontClass(Source).FontStrikeOut;
         FFontColor := TNZCSFontClass(Source).FontColor;
      end
      else begin
         if Source is TFont then begin
            FFontName := TFont(Source).Name;
            FFontSize := TFont(Source).Size;
            FFontBold := fsBold in TFont(Source).Style;
            FFontItalic := fsItalic in TFont(Source).Style;
            FFontUnderline := fsUnderline in TFont(Source).Style;
            FFontStrikeOut := fsStrikeout in TFont(Source).Style;
            FFontColor := TFont(Source).Color;
         end
         else begin
            inherited Assign(Source);
         end;
      end;
   finally
      EndUpdate;
      If Assigned(FOnChange) then
         FOnChange(self);
   end;
end;

Procedure TNZCSFontClass.BeginUpdate;
begin
   Inc(FUpdateLock);
end;

Procedure TNZCSFontClass.EndUpdate;
begin
   Dec(FUpdateLock);
end;

procedure TNZCSFontClass.SetFontBold(Value: Boolean);
begin
   If Value <> FFontBold then begin
      FFontBold := Value;
      If FUpdateLock = 0 then begin
         If Assigned(FOnChange) then
            FOnChange(self);
      end;
   end;
end;

procedure TNZCSFontClass.SetFontColor(Value: TColor);
begin
   If Value <> FFontColor then begin
      FFontColor := Value;
      If FUpdateLock = 0 then begin
         If Assigned(FOnChange) then
            FOnChange(self);
      end;
   end;
end;

procedure TNZCSFontClass.SetFontItalic(Value: Boolean);
begin
   If Value <> FFontItalic then begin
      FFontItalic := Value;
      If FUpdateLock = 0 then begin
         If Assigned(FOnChange) then
            FOnChange(self);
      end;
   end;
end;

procedure TNZCSFontClass.SetFontName(Value: String);
begin
   If Value <> FFontName then begin
      FFontName := Value;
      If FUpdateLock = 0 then begin
         If Assigned(FOnChange) then
            FOnChange(self);
      end;
   end;
end;

procedure TNZCSFontClass.SetFontSize(Value: Smallint);
begin
   If Value <> FFontSize then begin
      FFontSize := Value;
      If FUpdateLock = 0 then begin
         If Assigned(FOnChange) then
            FOnChange(self);
      end;
   end;
end;

procedure TNZCSFontClass.SetFontUnderline(Value: Boolean);
begin
   If Value <> FFontUnderline then begin
      FFontUnderline := Value;
      If FUpdateLock = 0 then begin
         If Assigned(FOnChange) then
            FOnChange(self);
      end;
   end;
end;

procedure TNZCSFontClass.SetFontStrikeOut(Value: Boolean);
begin
   If Value <> FFontStrikeOut then begin
      FFontStrikeOut := Value;
      If FUpdateLock = 0 then begin
         If Assigned(FOnChange) then
            FOnChange(self);
      end;
   end;
end;

{ TNZCSComponent }

constructor TNZCSComponent.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FFont := TNZCSFontClass.Create;
   FFont.OnChange := FontChanged;
end;

destructor TNZCSComponent.Destroy;
begin
   FFont.Free;
   inherited Destroy;
end;

procedure TNZCSComponent.FontChanged(Sender: TObject);
begin
   If assigned(FOnFontChange) then
      FOnFontChange(self);
end;

procedure Register;
begin
   RegisterComponents('NZCS', [TNZCSComponent]);
   RegisterPropertyEditor(TypeInfo(TNZCSFontClass), nil, '', TNZCSFontEditor);
   RegisterPropertyEditor(TypeInfo(string), TNZCSComponent, 'FontName', TFontNameProperty);
end;


end.
